@@ -1,15 +1,20 @@
 import base64
 import datetime
-import json
-import time
-import os
-import math
 import hashlib
-from typing import List, Dict, Any
+import json
+import logging
+import math
+import os
+import sys
+import time
+from typing import Any, Dict, List
+
 from rumpy.api.base import BaseAPI
 from rumpy.types.data import *
-from rumpy.utils import sha256, ts2datetime
+from rumpy.types.data import is_seed
+from rumpy.utils import sha256, timestamp_to_datetime
 
+logger = logging.getLogger(__name__)
 
 CHUNK_SIZE = 150 * 1024  # 150kb
 
@@ -26,6 +31,15 @@ def _nickname(pubkey, nicknames):
     return name
 
 
+def _trxs_unique(trxs: List):
+    """remove the duplicate trx from the trxs list"""
+    new = {}
+    for trx in trxs:
+        if trx["TrxId"] not in new:
+            new[trx["TrxId"]] = trx
+    return [new[i] for i in new]
+
+
 class Group(BaseAPI):
     def create(
         self,
@@ -53,39 +67,38 @@ class Group(BaseAPI):
         if consensus_type.lower() not in ("poa",):
             raise ValueError("consensus_type should be `poa` or `pos` or `pow`, but only `poa` is supported now.")
 
-        relay = {
+        payload = {
             "group_name": group_name,
             "consensus_type": consensus_type,
             "encryption_type": encryption_type,
             "app_key": app_key,
         }
 
-        return self._post("/group", relay)
+        return self._post("/group", payload)
 
-    def seed(self) -> Dict:
+    def seed(self, group_id=None) -> Dict:
         """get the seed of a group which you've joined in."""
-        if self.is_joined():
-            seed = self._get(f"/group/{self.group_id}/seed")
-            if "error" not in seed:
-                return seed
-        return {}
-
-    def is_seed(self, seed: Dict) -> bool:
-        try:
-            Seed(**seed)
-            return True
-        except Exception as e:
-            print(e)
-            return False
+        group_id = group_id or self.group_id
+        seed = self._get(f"/group/{group_id}/seed")
+        if "error" not in seed:
+            return seed
+        else:
+            raise ValueError(seed["error"])
 
     def info(self):
         """return group info,type: datacalss"""
-        if self.is_joined():
-            for info in self.node.groups():
-                if info["group_id"] == self.group_id:
-                    return GroupInfo(**info)
-        else:
-            raise ValueError(f"you are not in this group.{self.group_id}")
+        self.check_group_joined_as_required()
+
+        info = {}
+        for _info in self._client.node.groups():
+            if _info["group_id"] == self.group_id:
+                info = _info
+                break
+
+        if info.get("snapshot_info") is None:
+            info["snapshot_info"] = {}
+
+        return GroupInfo(**info)
 
     @property
     def pubkey(self):
@@ -99,42 +112,38 @@ class Group(BaseAPI):
     def eth_addr(self):
         return self.info().user_eth_addr
 
-    @property
-    def type(self):
-        self._check_group_id()
-        return self.seed().get("app_key")
-
     def join(self, seed: Dict):
         """join a group with the seed of the group"""
-        if not self.is_seed(seed):
+        if not is_seed(seed):
             raise ValueError("not a seed or the seed could not be identified.")
-        return self._post("/group/join", seed)
+        resp = self._post("/group/join", seed)
+        if "error" in resp:
+            if resp["error"] == "Group with same GroupId existed":
+                resp = True
+            else:
+                raise ValueError(resp["error"])
+        return resp
 
-    def is_joined(self) -> bool:
-        self._check_group_id()
-        if self.group_id in self.node.groups_id:
+    def is_joined(self, group_id=None) -> bool:
+        group_id = group_id or self.group_id
+        if group_id in self._client.node.groups_id:
             return True
         return False
 
-    def leave(self):
+    def leave(self, group_id=None):
         """leave a group"""
-        if self.is_joined():
-            return self._post("/group/leave", {"group_id": self.group_id})
+        group_id = group_id or self.group_id
+        return self._post("/group/leave", {"group_id": group_id})
 
-    def clear(self):
+    def clear(self, group_id=None):
         """clear data of a group"""
-        self._check_group_id()
-        return self._post("/group/clear", {"group_id": self.group_id})
+        group_id = group_id or self.group_id
+        return self._post("/group/clear", {"group_id": group_id})
 
-    def startsync(self):
+    def startsync(self, group_id=None):
         """触发同步"""
-        if self.is_joined():
-            return self._post(f"/group/{self.group_id}/startsync")
-
-    def content(self) -> List:
-        """get the content trxs of a group,return the list of the trxs data."""
-        if self.is_joined():
-            return self._get(f"/group/{self.group_id}/content") or []
+        group_id = group_id or self.group_id
+        return self._post(f"/group/{group_id}/startsync")
 
     def content_trxs(
         self,
@@ -157,15 +166,12 @@ class Group(BaseAPI):
             return []
 
         if trx_id:
-            apiurl = (
-                f"/group/{self.group_id}/content?num={num}&starttrx={trx_id}"
-                f"&reverse={str(is_reverse).lower()}&includestarttrx={str(is_include_starttrx).lower()}"
-            )
+            apiurl = f"/group/{self.group_id}/content?num={num}&starttrx={trx_id}&reverse={str(is_reverse).lower()}&includestarttrx={str(is_include_starttrx).lower()}"
         else:
             apiurl = f"/group/{self.group_id}/content?num={num}&reverse={str(is_reverse).lower()}"
 
         trxs = self._post(apiurl, api_base=self._client.api_base_app) or []
-        return self.trxs_unique(trxs)
+        return _trxs_unique(trxs)
 
     def _send(self, obj=None, sendtype=None, **kwargs) -> Dict:
         """return the {trx_id:trx_id} of this action if send successded
@@ -174,8 +180,8 @@ class Group(BaseAPI):
         sendtype: 发送类型, "Add"(发送内容), "Like"(点赞), "Dislike"(点踩)
         返回值 {"trx_id": "string"}
         """
-        relay = NewTrx(group_id=self.group_id, obj=obj, sendtype=sendtype, **kwargs).__dict__
-        return self._post("/group/content", relay)
+        payload = NewTrx(group_id=self.group_id, obj=obj, sendtype=sendtype, **kwargs).__dict__
+        return self._post("/group/content", payload)
 
     def like(self, trx_id: str) -> Dict:
         return self._send(trx_id=trx_id, sendtype="Like")
@@ -208,7 +214,11 @@ class Group(BaseAPI):
             file_obj.seek(i * CHUNK_SIZE)
             ibytes = file_obj.read(current_size)
             fileinfo["segments"].append({"id": f"seg-{i+1}", "sha256": sha256(ibytes)})
-            obj = FileObj(name=f"seg-{i + 1}", content=ibytes, mediaType="application/octet-stream")
+            obj = FileObj(
+                name=f"seg-{i + 1}",
+                content=ibytes,
+                mediaType="application/octet-stream",
+            )
             objs.append(obj)
 
         content = json.dumps(fileinfo).encode()
@@ -220,7 +230,8 @@ class Group(BaseAPI):
 
     def upload(self, file_path):
         if not os.path.isfile(file_path):
-            return print(f"{file_path} is not a file.")
+            logger.info(f"{sys._getframe().f_code.co_name}, {file_path} is not a file.")
+            return
         for obj in self._file_to_objs(file_path):
             self._send(obj=obj, sendtype="Add")
 
@@ -229,9 +240,9 @@ class Group(BaseAPI):
         infos = []
         filetrxs = []
         for trx in trxs:
-            if trx["Content"]["name"] == "fileinfo":
+            if trx["Content"].get("name") == "fileinfo":
                 info = eval(base64.b64decode(trx["Content"]["file"]["content"]).decode("utf-8"))
-                print(info)
+                logger.debug(f"{sys._getframe().f_code.co_name}, {info}")
                 infos.append(info)
             if trx["Content"].get("type") == "File":
                 filetrxs.append(trx)
@@ -241,7 +252,8 @@ class Group(BaseAPI):
 
         ifilepath = os.path.join(file_dir, info["name"])
         if os.path.exists(ifilepath):
-            return print(ifilepath, "file exists.")
+            logger.info(f"{sys._getframe().f_code.co_name}, file exists {ifilepath}")
+            return
 
         # _check_trxs
         right_shas = [i["sha256"] for i in info["segments"]]
@@ -257,11 +269,11 @@ class Group(BaseAPI):
 
         for seg in info["segments"]:
             if seg["sha256"] not in contents:
-                print(seg, "trx is not exists...")
+                logger.info(f"{sys._getframe().f_code.co_name}, " + json.dumps(seg) + ", trx is not exists...")
                 flag = False
                 break
             if contents[seg["sha256"]]["Content"].get("name") != seg["id"]:
-                print(seg, "name is different...")
+                logger.info(f"{sys._getframe().f_code.co_name}, " + json.dumps(seg) + ", name is different...")
                 flag = False
                 break
 
@@ -271,12 +283,14 @@ class Group(BaseAPI):
                 content = base64.b64decode(contents[seg["sha256"]]["Content"]["file"]["content"])
                 ifile.write(content)
             ifile.close()
-            print(ifilepath, "downloaded!")
+            logger.info(f"{sys._getframe().f_code.co_name}, {ifilepath}, downloaded!")
 
     def download(self, file_dir):
+        logger.debug(f"download file to dir {file_dir} start...")
         infos, trxs = self._file_infos()
         for info in infos:
             self._down_load(file_dir, info, trxs)
+        logger.debug(f"download file to dir {file_dir} done")
 
     def send_note(self, **kwargs):
         return self._send(sendtype="Add", objtype="Note", **kwargs)
@@ -310,7 +324,7 @@ class Group(BaseAPI):
 
     def block(self, block_id: str):
         """get the info of a block in a group"""
-        self._check_group_id()
+        self.check_group_id_as_required()
         return self._get(f"/block/{self.group_id}/{block_id}")
 
     def is_owner(self) -> bool:
@@ -369,30 +383,24 @@ class Group(BaseAPI):
             return "text_only"
         return trxtype.lower()  # "like","dislike","other"
 
-    def trx(self, trx_id: str = None):
-        try:
-            resp = self.content_trxs(trx_id=trx_id, num=1, is_include_starttrx=True)
-            if len(resp) > 1:
-                print("something is error", resp)
-            elif len(resp) == 0:
-                raise ValueError(f"nothing got. {resp} {trx_id} {self.group_id}")
-            else:
-                return resp[0]
-        except Exception as e:
-            print(e)
-            return self._get(f"/trx/{self.group_id}/{trx_id}")
-        return {"error": "nothing got."}
-
-    def trxs_unique(self, trxs: List):
-        """remove the duplicate trx from the trxs list"""
-        new = {}
-        for trx in trxs:
-            if trx["TrxId"] not in new:
-                new[trx["TrxId"]] = trx
-        return [new[i] for i in new]
+    def trx(self, trx_id: str):
+        """get trx data by trx_id"""
+        self.check_group_id_as_required()
+        data = {}
+        trxs = self.content_trxs(trx_id=trx_id, num=1, is_include_starttrx=True)
+        if len(trxs) > 1:
+            raise ValueError(f"{len(trxs)} trxs got from group: <{self.group_id}> with trx: <{trx_id}>.")
+        elif len(trxs) == 1:
+            data = trxs[0]
+        else:
+            data = self._get(f"/trx/{self.group_id}/{trx_id}")
+            logger.info(f"data is encrypted for trx: <{trx_id}> of group: <{self.group_id}>.")
+        if not data:
+            logger.warning(f"data is empty for trx: <{trx_id}> of group: <{self.group_id}>.")
+        return data
 
     def pubqueue(self):
-        self._check_group_id()
+        self.check_group_id_as_required()
         resp = self._get(f"/group/{self.group_id}/pubqueue")
         return resp.get("Data")
 
@@ -400,16 +408,17 @@ class Group(BaseAPI):
         return self._post("/trx/ack", {"trx_ids": trx_ids})
 
     def autoack(self):
-        self._check_group_id()
+        self.check_group_id_as_required()
         tids = [i["Trx"]["TrxId"] for i in self.pubqueue() if i["State"] == "FAIL"]
         return self.ack(tids)
 
-    def get_users_profiles(self, users_data: Dict, types=("name", "image", "wallet")) -> Dict:
-        """Returns:
+    def get_users_profiles(self, users_data: Dict = {}, types=("name", "image", "wallet")) -> Dict:
+        """update users_data and returns it.
         {
-            group_id:  "",
-            group_name: "",
-            trx_id: "",
+            group_id:  "", # the group_id
+            group_name: "", # the group_name
+            trx_id: "", # the trx_id of groups progress
+            trx_timestamp:"",
             update_at: "",
             data:{ pubkey:{
                 name:"",
@@ -419,23 +428,30 @@ class Group(BaseAPI):
             }
         }
         """
+        # check group_id
+        group_id = users_data.get("group_id", self.group_id)
+        if group_id != self.group_id:
+            logger.warning(
+                f"get_users_profiles: group_id is different. client.group_id:{self.group_id}, data.group_id:{group_id}"
+            )
+            return users_data
 
         # get new trxs from the trx_id
-        trx_id = users_data.get("trx_id")
-        trxs = self.group.all_content_trxs(trx_id=trx_id)
+        trx_id = users_data.get("trx_id", None)
+        trxs = self.all_content_trxs(trx_id=trx_id)
+
+        if len(trxs) == 0:
+            logger.debug(f"get_users_profiles: got 0 new trxs. get content started from trx_id:{trx_id}.")
+            return users_data
 
         # update trx_id: to record the progress to get new trxs
-        if len(trxs) > 0:
-            to_tid = trxs[-1]["TrxId"]
-        else:
-            to_tid = trx_id
+
         users_data.update(
             {
                 "group_id": self.group_id,
-                "group_name": self.group.seed().get("group_name"),
-                "trx_id": to_tid,
-                "trx_timestamp": str(ts2datetime(self.group.trx(to_tid).get("TimeStamp"))),
-                "update_at": str(datetime.datetime.now()),
+                "group_name": self.seed().get("group_name"),
+                "trx_id": trxs[-1]["TrxId"],
+                "trx_timestamp": str(timestamp_to_datetime(trxs[-1].get("TimeStamp", ""))),
             }
         )
 
@@ -451,31 +467,7 @@ class Group(BaseAPI):
             for key in trx["Content"]:
                 if key in types:
                     users[pubkey].update({key: trx["Content"][key]})
-        users_data.update({"data": users})
-        return users_data
-
-    def update_profiles(
-        self,
-        users_data=None,
-        users_profiles_file=None,
-        datadir=None,
-        types=("name", "wallet", "image"),
-    ):
-        from officy import JsonFile
-
-        if users_data:
-            return self.get_users_profiles(users_data, types)
-
-        filename = f"users_profiles_group_{self.group_id}.json"
-        if datadir:
-            users_profiles_file = os.path.join(datadir, filename)
-        else:
-            users_profiles_file = users_profiles_file or filename
-
-        users_data = JsonFile(users_profiles_file).read({})
-        users_data = self.get_users_profiles(users_data, types)
-
-        JsonFile(users_profiles_file).write(users_data)
+        users_data.update({"data": users, "update_at": str(datetime.datetime.now())})
         return users_data
 
     def trx_to_newobj(self, trx, nicknames, refer_trx=None):
@@ -486,7 +478,7 @@ class Group(BaseAPI):
             nicknames (dict): the nicknames data of the group
 
         Returns:
-            obj: object of NewTrx,can be used as: self.group.send_note(obj=obj).
+            obj: object of NewTrx,can be used as: self.send_note(obj=obj).
             result: True,or False, if True, the obj can be send to chain.
         """
 
@@ -513,19 +505,22 @@ class Group(BaseAPI):
                 lines.append(text)
                 obj["image"].extend(img)
 
-                t = self.group.trx_type(trx)
+                t = self.trx_type(trx)
                 refer_tid = None
                 _info = {"like": "赞", "dislike": "踩"}
                 if t == "announce":
                     lines.insert(0, f"处理了链上请求。")
                 elif t in _info:
                     refer_tid = trx["Content"]["id"]
-                    refer_pubkey = self.group.trx(refer_tid).get("Publisher", "")
-                    lines.insert(0, f"点{_info[t]}给 `{_nickname( refer_pubkey,nicknames)}` 所发布的内容：")
+                    refer_pubkey = self.trx(refer_tid).get("Publisher", "")
+                    lines.insert(
+                        0,
+                        f"点{_info[t]}给 `{_nickname( refer_pubkey,nicknames)}` 所发布的内容：",
+                    )
                 elif t == "reply":
                     lines.insert(0, f"回复说：")
                     refer_tid = trx["Content"]["inreplyto"]["trxid"]
-                    refer_pubkey = self.group.trx(refer_tid).get("Publisher", "")
+                    refer_pubkey = self.trx(refer_tid).get("Publisher", "")
                     lines.append(f"\n回复给 `{_nickname(refer_pubkey,nicknames)}` 所发布的内容：")
                 else:
                     if text and img:
@@ -537,17 +532,16 @@ class Group(BaseAPI):
 
                 if refer_tid:
 
-                    refer_trx = refer_trx or self.group.trx(refer_tid)
+                    refer_trx = refer_trx or self.trx(refer_tid)
                     if "Content" in refer_trx:
                         refer_text = refer_trx["Content"].get("content", "")
                         refer_img = refer_trx["Content"].get("image", [])
                         lines.append(_quote(refer_text))
                         obj["image"].extend(refer_img)
         else:
-            print(trx)
             return None, False
 
-        obj["content"] = f'{ts2datetime(trx.get("TimeStamp"))}' + " " + "\n".join(lines)
+        obj["content"] = f'{timestamp_to_datetime(trx.get("TimeStamp"))}' + " " + "\n".join(lines)
         obj["image"] = obj["image"][:4]
         obj = {key: obj[key] for key in obj if obj[key]}
 
